var map, geocoder, providers, popup;
var lastTopFeature, lastSearchedPoint;

// Global object to keep track of filter values.
var filters = {
  population_served: null,
  co_occurring: null,
  level_of_care: null,
  insurance_payment: null,
  treatments_offered: null,
  evidence_based: null,
};

/**
 * Mapping tables for converting the UI's filter names to key-value
 * pairs compatible with the Salesforce test TP data. These won't be
 * needed in production if we use a different data source.
 */
var sf_conversion = {
  'cat_sf': {
    'population_served': 'Populations_Served__c',
    'co_occurring': 'Co_Occuring_Psychiatric_Issues__c',
    'level_of_care': 'Levels_of_Care_Setting__c',
    'insurance_payment': 'Insurance_Payment_Options__c',
    'treatments_offered': 'Treatments_Offered__c',
    'evidence_based': 'Evidence_Based_Treatment__c',
  },

  'opt_sf': {
    'population_served' : {
      'population_an': 'Anorexia Nervosa',
      'population_bn': 'Bulimia Nervosa',
      'population_bed': 'Binge Eating Disorder',
      'population_osfed': 'OSFED',
      'population_pica': 'Pica',
      'population_diabulimia': 'Diabulimia',
      'population_male': 'Male',
      'population_female': 'Female',
      'population_children': 'Children',
      'population_adolescent': 'Adolescent',
      'population_adult': 'Adult',
      'population_geriatrics': 'Geriatrics',
      'population_lgbtq': 'LGBTQ',
    },
    'co_occurring': {
      'cooccurring_substance': 'Substance Abuse',
      'cooccurring_autism': 'Autism Spectrum Disorders',
      'cooccurring_schizophrenia': 'Schizophrenia',
      'cooccurring_mood': 'Mood Disorders',
      'cooccurring_anxiety': 'Anxiety Disorders',
      'cooccurring_dissociative': 'Dissociative Disorders',
      'cooccurring_ocd': 'Obsessive Compulsive Disorder',
      'cooccurring_trauma': 'Trauma',
      'cooccurring_depression': 'Depression',
      'cooccurring_selfharm': 'Self-Harm',
    },
    'level_of_care': {
      'setting_hospital': 'Hospital',
      'setting_residential': 'Residential',
      'setting_partialhospitalization': 'Partial Hospitalization',
      'setting_specialized': 'Specialized Program',
      'setting_outpatient': 'Intensive Outpatient',
      'setting_private': 'Private Practice',
      'setting_academic': 'Academic/Research/Clinical Institution',
      'setting_psychiatric': 'Psychiatric',
    },
    'insurance_payment': {
      'payment_medicare': 'Medicare',
      'payment_medicaid': 'Medicaid',
      'payment_tricare': 'Tricare',
      'payment_private': 'Private Insurance',
      'payment_sliding': 'Sliding Scale',
      'payment_scholarships': 'Scholarships',
      'payment_cash': 'Cash',
      'payment_credit': 'Credit',
      'payment_check': 'Check',
    },
    'treatments_offered': {
      'treatment_individual': 'Individual Therapy',
      'treatment_family': 'Family Therapy',
      'treatment_couples': 'Couples Therapy',
      'treatment_group': 'Group Therapy',
      'treatment_psychoanalytic': 'Psychoanalytic',
      'treatment_psychodynamic': 'Psychodynamic',
      'treatment_12_step': '12-step',
      'treatment_faith': 'Faith-based',
      'treatment_meal': 'Meal Plan',
      'treatment_intuitive': 'Intuitive Eating',
      'treatment_mindfulness': 'Mindfulness',
      'treatment_nutrition': 'Nutrition/Dietetics',
      'treatment_phone': 'Phone Therapy',
      'treatment_video': 'Video Therapy',
    },
    'evidence_based': {
      'evidence_cbt': 'Cognitive Behavioral Therapy (CBT)',
      'evidence_dbt': 'Dialectical Behavioral Therapy (DBT)',
      'evidence_ipt': 'Interpersonal Therapy (IPT)',
      'evidence_fbt': 'Family Based Treatment (FBT)',
      'evidence_medicine': 'Psychopharmacology (Medicine)',
    },
  },
}

/**
 * Return true exactly when the given feature meets the criteria defined
 * by the filters currently applied in the UI.
 */
var passesFilter = function(feature) {
  if (feature == undefined) return false;

  var sf_option, sf_category;
  for (var filter in filters) {
    sf_category = sf_conversion['cat_sf'][filter];
    for (var i = 0; i < filters[filter].length; i++) {
      sf_option = sf_conversion['opt_sf'][filter][filters[filter][i]];
      if (feature.properties[sf_category].indexOf(sf_option) < 0) {
        return false;
      }
    }
  }

  return true;
};

/**
 * Update the state of the global filter object based on the
 * state of the DOM filters.
 */
var updateFilters = function() {
  var dfd = new $.Deferred();
  filters.population_served = $('#population-served').val();
  filters.co_occurring = $('#co-occurring').val();
  filters.level_of_care = $('#level-of-care').val();
  filters.insurance_payment = $('#insurance-payment').val();
  filters.treatments_offered = $('#treatments-offered').val();
  filters.evidence_based = $('#evidence-based').val();
  dfd.resolve();
  return dfd.promise();
};

var attachEventListeners = function() {
  // On filter change, refresh the providers on the map and the listings.
  $('.select').on('select2:select select2:unselect', function() {
    updateFilters().then(function() {
      buildLocationList();
    });
  });

  //geocoder.on('clear', function() { partialMapShown(false); });

  // Resize map on map toggle
  /*
  $('.show-btn').click(function() {
    $('.content').toggleClass('map-shown').promise().done(function() {
      // Make sure map fills the allotted area
      if ($('.content').hasClass('map-shown')) {
        partialMapShown(false);
        map.resize();
        scaleMapToPoints(lastTopFeature, lastSearchedPoint);
      }
    });
  });
  */

  // Resize map on map toggle
  $('.show-btn').click(function() {
    $('.content').toggleClass('map-shown').promise().done(function() {
      // Make sure map fills the allotted area
      if ($('.content').hasClass('map-shown')) {
        partialMapShown(false);
        //map.resize();
        scaleMapToPoints(lastTopFeature, lastSearchedPoint);
      }
    });
  });

  // Open and close filters.
  $('.filters-toggle').click(function(e) {
    e.preventDefault();
    $('.content').toggleClass('filters-open');
  });
  $('#apply-filters').click(function(e) {
    e.preventDefault();
    $('.content').toggleClass('filters-open');
  });
};

$(document).ready(function() {
  // Use select2 for dynamic dropdown filters.
  $.each($('.select'), function(i, select) {
    $(select).select2({
      width: "100%",
      placeholder: $(select).attr('data-placeholder'),
    });
  });

  updateFilters();
  attachEventListeners();
});

/**
 * With no arguments, switch the map view mode to partial. If "show"
 * is false, switch from partial to full mode.
 */
var partialMapShown = function (show = true) {
  if (show)
    $('.content').addClass('partial-map-show');
  else
    $('.content').removeClass('partial-map-show');
};

/**
 * Given a feature object, generate markup for a text listing and append
 * it to the DOM text feature list.
 */
var addFeatureToListings = function(index) {
  // Create container div
  var listing = document.getElementById("listings-items")
    .appendChild(document.createElement("div"));
  listing.className = "item";
  listing.id = "listing-" + index;

  // Link on the listing's street address
  var link = listing.appendChild(document.createElement("a"));
  link.href = "#";
  link.className = "title";
  link.dataPosition = index;
  link.innerHTML = providers[index].properties.Name__c;

  // Add div for details
  var details = listing.appendChild(document.createElement("div"));
  details.innerHTML = providers[index].properties.Street_Address__c + '<br />';
  details.innerHTML += providers[index].properties.City__c + ', ' +
    providers[index].properties.State__c;
  if (providers[index].properties.Phone_Number__c)
    details.innerHTML += " &middot; " + providers[index].properties.Phone_Number__c;
  if (providers[index].properties.distance) {
    var roundedDistance = Math.round(providers[index].properties.distance * 100) / 100;
    details.innerHTML += "<p class='result-distance'><strong>" + roundedDistance +
      " miles away</strong></p>";
  }

  // Add listener for link click event.
  link.addEventListener("click", function() {
    var clickedListing = providers[this.dataPosition];
    flyToFeature(clickedListing); // Fly to closest provider
    selectFeature(clickedListing); // Create popup on that provider

    // Apply class "active" to the clicked provider
    var activeItem = document.getElementsByClassName("active");
    if (activeItem[0]) activeItem[0].classList.remove("active");
    this.parentNode.classList.add("active");

    partialMapShown();
  });
}


/**
 * Refresh the visibility of each map marker and the text TP list based
 * on current filter values.
 */
var buildLocationList = function() {
  // Clear the listings div
  var listings = document.getElementById('listings-items');
  while (listings.firstChild)
    listings.removeChild(listings.firstChild);

  var maxListings = 5; // Maximum number to show in text list

  // Update map classes based on filters
  /*
  var filterClass = '';
  $('.map-container').attr('class', 'map-container');
  for (var filter in filters) {
    sf_category = sf_conversion['cat_sf'][filter];
    for (var i = 0; i < filters[filter].length; i++) {
      sf_option = sf_conversion['opt_sf'][filter][filters[filter][i]];

      filterClass = sf_category + '--' + sf_option.replace(/\W/g, '');
      $('.map-container').addClass(filterClass);
    }
  }
  */

  var numAlreadyListed = 0;
  for (var i = 0; i < providers.length; i++) {
    // Ignore any features unsuccessfully geocoded.
    if (!providers[i].geometry.coordinates[0] &&
        !providers[i].geometry.coordinates[1])
      continue;

    // If number of listings exceeds the maximum, stop writing to the text 
    // list, but continue adding markers to the map.
    if (passesFilter(providers[i]) && numAlreadyListed < maxListings) {
      addFeatureToListings(i);
      numAlreadyListed++;
    } else {
      // @TODO remove marker
    }
  }
}


/**
 * Scale the map to fit both the search point and the provider
 * closest to it.
 */
var scaleMapToPoints = function (featureID, searchResult) {
  var lats = [
    providers[featureID].geometry.coordinates[1],
    searchResult[1]
  ];
  var lons = [
    providers[featureID].geometry.coordinates[0],
    searchResult[0]
  ];

  var sortedLons = lons.sort(function(a, b) {
    if (a > b) return 1;
    if (a.distance < b.distance) return -1;
    return 0;
  });
  var sortedLats = lats.sort(function(a, b) {
    if (a > b) return 1;
    if (a.distance < b.distance) return -1;
    return 0;
  });

  map.fitBounds({
    west: sortedLons[0],
    south: sortedLats[0],
    east: sortedLons[1],
    north: sortedLats[1]
  }, 100);
}


var geocoderSearchCallback = function () {
  $('.content').addClass('has-results');

  var address = document.getElementById('address').value;
  geocoder.geocode({'address': address}, function(results, status) {
    if (status === 'OK') {
      map.setCenter(results[0].geometry.location);
      var marker = new google.maps.Marker({
        map: map,
        position: results[0].geometry.location,
        icon: {
          path: google.maps.SymbolPath.CIRCLE,
          scale: 8.5,
          fillColor: "#007cbf",
          fillOpacity: 1,
          strokeWeight: 1,
        },
      });

      var searchResult = [marker.position.lng(), marker.position.lat()];

      // Compute each provider's distance from the search point.
      providers.forEach(function(provider) {
        Object.defineProperty(provider.properties, "distance", {
          value: turf.distance(
            searchResult,
            provider.geometry,
            { units: "miles" },
          ),
          writable: true,
          enumerable: true,
          configurable: true,
        });
      });

      // Sort providers by ascending distance from search point.
      providers.sort(function(a, b) {
        if (a.properties.distance > b.properties.distance)
          return 1;
        if (a.properties.distance < b.properties.distance)
          return -1;
        return 0;
      });

      // Find the index of the closest feature that passes the filters.
      var topFeatureId = 0;
      /*
      for (var i = 0; i < providers.features.length; i++) {
        if (passesFilter(providers.features[i])) {
          topFeatureId = i;
          break;
        }
      }
      */

      //if (topFeatureId > 0 || passesFilter(providers.features[0])) { 
        // Zoom and display popup on closest filter-appropriate feature.
        scaleMapToPoints(topFeatureId, searchResult);
        lastSearchedPoint = searchResult;
        lastTopFeature = topFeatureId;
        //selectFeature(providers.features[topFeatureId]);
        buildLocationList(providers);
        /*
      } else {
        $('#listings').html('<p>No results found. Try removing some filters.</p>');
      }
      */

    } else {
      alert('Geocode was not successful for the following reason: ' + status);
    }
  });
};

var flyToFeature = function(feature) {
  if (map.getZoom() < 13) map.setZoom(13);
  map.panTo({
    lng: feature.geometry.coordinates[0],
    lat: feature.geometry.coordinates[1]
  });
};

var selectFeature = function(feature) {
  popup.setContent("<h4>" + feature.properties.Name__c + "</h4>");
  popup.open(map, feature.marker);
};

var featureDrawMarker = function(feature) {
  // Ignore any features unsuccessfully geocoded
  if (!feature.geometry.coordinates[0] && !feature.geometry.coordinates[1])
    return;

  feature.marker = new google.maps.Marker({
    position: { lat: feature.geometry.coordinates[1], lng: feature.geometry.coordinates[0] },
    map: map,
  });

  feature.marker.addListener("click", function(e) {
    flyToFeature(feature);
    selectFeature(feature);
  });
};

var initializeMapUI = function(featureData) {
  providers = featureData;
  popup = new google.maps.InfoWindow();
  providers.forEach(function(feature) { featureDrawMarker(feature); });
};

var initMap = function () {
  map = new google.maps.Map(document.getElementById('map'), {
    center: { lng: -94, lat: 42 },
    zoom: 4,
  });
  geocoder = new google.maps.Geocoder();
  document.getElementById('address').addEventListener('keypress', function (e) {
    var key = e.which || e.keyCode;
    if (key === 13) geocoderSearchCallback();
  });

  $.getJSON('sf_geocoded_3.json', function(data) {
    initializeMapUI(data);
  });
};
